use std::str::FromStr;

use palette::{
    color_theory::{Analogous, Complementary, SplitComplementary, Tetradic, Triadic},
    FromColor, Oklch, ShiftHue, Srgb,
};
use wasm_bindgen::prelude::wasm_bindgen;

use crate::errors;

#[wasm_bindgen]
pub fn shift_hue(color: &str, degree: f32) -> Result<String, errors::ColorError> {
    let origin_color = Srgb::from_str(color)
        .map_err(|_| errors::ColorError::UnrecogniazedRGB(color.to_string()))?
        .into_format::<f32>();
    let oklch = Oklch::from_color(origin_color);
    let shifted_color = oklch.shift_hue(degree);
    let srgb = Srgb::from_color(shifted_color);
    Ok(format!("{:x}", srgb.into_format::<u8>()))
}

#[wasm_bindgen]
pub fn analogous_30(color: &str) -> Result<Vec<String>, errors::ColorError> {
    let origin_color = Srgb::from_str(color)
        .map_err(|_| errors::ColorError::UnrecogniazedRGB(color.to_string()))?
        .into_format::<f32>();
    let oklch = Oklch::from_color(origin_color);
    let (color_n30, color_p30) = oklch.analogous();
    let srgb_n30 = Srgb::from_color(color_n30);
    let srgb_p30 = Srgb::from_color(color_p30);
    Ok(vec![
        format!("{:x}", srgb_n30.into_format::<u8>()),
        format!("{:x}", srgb_p30.into_format::<u8>()),
    ])
}

#[wasm_bindgen]
pub fn analogous_60(color: &str) -> Result<Vec<String>, errors::ColorError> {
    let origin_color = Srgb::from_str(color)
        .map_err(|_| errors::ColorError::UnrecogniazedRGB(color.to_string()))?
        .into_format::<f32>();
    let oklch = Oklch::from_color(origin_color);
    let (color_n60, color_p60) = oklch.analogous_secondary();
    let srgb_n60 = Srgb::from_color(color_n60);
    let srgb_p60 = Srgb::from_color(color_p60);
    Ok(vec![
        format!("{:x}", srgb_n60.into_format::<u8>()),
        format!("{:x}", srgb_p60.into_format::<u8>()),
    ])
}

#[wasm_bindgen]
pub fn complementary(color: &str) -> Result<String, errors::ColorError> {
    let origin_color = Srgb::from_str(color)
        .map_err(|_| errors::ColorError::UnrecogniazedRGB(color.to_string()))?
        .into_format::<f32>();
    let oklch = Oklch::from_color(origin_color);
    let complementary_color = oklch.complementary();
    let srgb = Srgb::from_color(complementary_color);
    Ok(format!("{:x}", srgb.into_format::<u8>()))
}

#[wasm_bindgen]
pub fn split_complementary(color: &str) -> Result<Vec<String>, errors::ColorError> {
    let origin_color = Srgb::from_str(color)
        .map_err(|_| errors::ColorError::UnrecogniazedRGB(color.to_string()))?
        .into_format::<f32>();
    let oklch = Oklch::from_color(origin_color);
    let (color_p150, color_p210) = oklch.split_complementary();
    let srgb_p150 = Srgb::from_color(color_p150);
    let srgb_p210 = Srgb::from_color(color_p210);
    Ok(vec![
        format!("{:x}", srgb_p150.into_format::<u8>()),
        format!("{:x}", srgb_p210.into_format::<u8>()),
    ])
}

#[wasm_bindgen]
pub fn tetradic(color: &str) -> Result<Vec<String>, errors::ColorError> {
    let origin_color = Srgb::from_str(color)
        .map_err(|_| errors::ColorError::UnrecogniazedRGB(color.to_string()))?
        .into_format::<f32>();
    let oklch = Oklch::from_color(origin_color);
    let (color_p90, color_p180, color_p270) = oklch.tetradic();
    let srgb_p90 = Srgb::from_color(color_p90);
    let srgb_p180 = Srgb::from_color(color_p180);
    let srgb_p270 = Srgb::from_color(color_p270);
    Ok(vec![
        format!("{:x}", srgb_p90.into_format::<u8>()),
        format!("{:x}", srgb_p180.into_format::<u8>()),
        format!("{:x}", srgb_p270.into_format::<u8>()),
    ])
}

#[wasm_bindgen]
pub fn triadic(color: &str) -> Result<Vec<String>, errors::ColorError> {
    let origin_color = Srgb::from_str(color)
        .map_err(|_| errors::ColorError::UnrecogniazedRGB(color.to_string()))?
        .into_format::<f32>();
    let oklch = Oklch::from_color(origin_color);
    let (color_p120, color_p240) = oklch.triadic();
    let srgb_p120 = Srgb::from_color(color_p120);
    let srgb_p240 = Srgb::from_color(color_p240);
    Ok(vec![
        format!("{:x}", srgb_p120.into_format::<u8>()),
        format!("{:x}", srgb_p240.into_format::<u8>()),
    ])
}
